@@ -19,17 +19,17 @@
 use std;
 use ffi;
 use python::{Python, PythonObject, PythonObjectWithCheckedDowncast, ToPythonPointer};
-use objects::{exc, PyObject, PyBool, PyTuple};
-use err::{self, PyErr, PyResult};
+use objects::PyObject;
+use err::PyResult;
 
-/// Conversion trait that allows various objects to be converted into python objects.
+/// Conversion trait that allows various objects to be converted into Python objects.
 pub trait ToPyObject<'p> {
     type ObjectType : PythonObject<'p> = PyObject<'p>;
 
-    /// Converts self into a python object.
+    /// Converts self into a Python object.
     fn to_py_object(&self, py: Python<'p>) -> Self::ObjectType;
 
-    /// Converts self into a python object.
+    /// Converts self into a Python object.
     ///
     /// May be more efficient than `to_py_object` in some cases because
     /// it can move out of the input object.
@@ -39,16 +39,16 @@ pub trait ToPyObject<'p> {
         self.to_py_object(py)
     }
 
-    /// Converts self into a python object and calls the specified closure
-    /// on the native FFI pointer underlying the python object.
+    /// Converts self into a Python object and calls the specified closure
+    /// on the native FFI pointer underlying the Python object.
     ///
     /// May be more efficient than `to_py_object` because it does not need
-    /// to touch any reference counts when the input object already is a python object.
+    /// to touch any reference counts when the input object already is a Python object.
     #[inline]
     fn with_borrowed_ptr<F, R>(&self, py: Python<'p>, f: F) -> R
       where F: FnOnce(*mut ffi::PyObject) -> R {
-        let obj = self.to_py_object(py);
-        f(ToPythonPointer::as_ptr(&obj))
+        let obj = self.to_py_object(py).into_object();
+        f(obj.as_ptr())
     }
 
     // FFI functions that accept a borrowed reference will use:
@@ -58,7 +58,7 @@ pub trait ToPyObject<'p> {
     // 2) input is PyObject
     //   -> with_borrowed_ptr() just forwards to the closure
     // 3) input is &str, int, ...
-    //   -> to_py_object() allocates new python object; FFI call happens; PyObject::drop() calls Py_DECREF()
+    //   -> to_py_object() allocates new Python object; FFI call happens; PyObject::drop() calls Py_DECREF()
     
     // FFI functions that steal a reference will use:
     //   let input = try!(input.into_py_object()); ffi::Call(input.steal_ptr())
@@ -67,18 +67,18 @@ pub trait ToPyObject<'p> {
     // 2) input is PyObject
     //   -> into_py_object() is no-op
     // 3) input is &str, int, ...
-    //   -> into_py_object() allocates new python object
+    //   -> into_py_object() allocates new Python object
 }
 
-/// FromPyObject is implemented by various types that can be extracted from a python object.
-pub trait FromPyObject<'p, 's> {
-    fn from_py_object(s: &'s PyObject<'p>) -> PyResult<'p, Self>;
+/// FromPyObject is implemented by various types that can be extracted from a Python object.
+pub trait FromPyObject<'p> {
+    fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, Self>;
 }
 
 // PyObject, PyModule etc.
 // We support FromPyObject and ToPyObject for owned python references.
-// This allows using existing python objects in code that generically expects a value
-// convertible to a python object.
+// This allows using existing Python objects in code that generically expects a value
+// convertible to a Python object.
 
 /// Identity conversion: allows using existing `PyObject` instances where
 /// `ToPyObject` is expected.
@@ -91,7 +91,7 @@ impl <'p, 's> ToPyObject<'p> for PyObject<'s> {
     }
 
     #[inline]
-    fn into_py_object(self, py: Python<'p>) -> PyObject<'p> {
+    fn into_py_object(self, _py: Python<'p>) -> PyObject<'p> {
         // Transmute the lifetime.
         // This is safe, because both lifetime variables represent the same lifetime:
         // that of the python GIL acquisition.
@@ -99,40 +99,40 @@ impl <'p, 's> ToPyObject<'p> for PyObject<'s> {
     }
 
     #[inline]
-    fn with_borrowed_ptr<F, R>(&self, py: Python<'p>, f: F) -> R
+    fn with_borrowed_ptr<F, R>(&self, _py: Python<'p>, f: F) -> R
       where F: FnOnce(*mut ffi::PyObject) -> R {
         f(self.as_ptr())
     }
 }
 
-impl <'p, 's, T> FromPyObject<'p, 's> for T where T: PythonObjectWithCheckedDowncast<'p> {
+impl <'p, T> FromPyObject<'p> for T where T: PythonObjectWithCheckedDowncast<'p> {
     #[inline]
-    fn from_py_object(s : &'s PyObject<'p>) -> PyResult<'p, T> {
+    fn from_py_object(s : &PyObject<'p>) -> PyResult<'p, T> {
         Ok(try!(s.clone().cast_into()))
     }
 }
 
 // &PyObject, &PyModule etc.
 // We support FromPyObject and ToPyObject for borrowed python references.
-// This allows using existing python objects in code that generically expects a value
-// convertible to a python object.
-impl <'p, 's, T> ToPyObject<'p> for &'s T where T : ToPyObject<'p> {
+// This allows using existing Python objects in code that generically expects a value
+// convertible to a Python object.
+impl <'p, 's, T: ?Sized> ToPyObject<'p> for &'s T where T: ToPyObject<'p> {
     type ObjectType = T::ObjectType;
 
     #[inline]
     fn to_py_object(&self, py: Python<'p>) -> T::ObjectType {
-        (**self).to_py_object(py)
+        <T as ToPyObject>::to_py_object(*self, py)
     }
 
     #[inline]
     fn into_py_object(self, py: Python<'p>) -> T::ObjectType {
-        (*self).to_py_object(py)
+        <T as ToPyObject>::to_py_object(self, py)
     }
 
     #[inline]
     fn with_borrowed_ptr<F, R>(&self, py: Python<'p>, f: F) -> R
       where F: FnOnce(*mut ffi::PyObject) -> R {
-        (**self).with_borrowed_ptr(py, f)
+        <T as ToPyObject>::with_borrowed_ptr(*self, py, f)
     }
 }
 
