@@ -16,14 +16,12 @@
 // OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-use std;
-use std::{fmt, string};
-use std::borrow::Cow;
+use std::fmt;
 use std::cmp::Ordering;
 use ffi;
 use libc;
-use python::{Python, PythonObject, PythonObjectWithCheckedDowncast, ToPythonPointer};
-use objects::{PyObject, PyTuple, PyDict};
+use python::{PythonObject, ToPythonPointer};
+use objects::{PyObject, PyTuple, PyDict, PyString};
 use conversion::ToPyObject;
 use err::{PyErr, PyResult, result_from_owned_ptr, error_on_minusone};
 
@@ -32,7 +30,7 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     /// Determines whether this object has the given attribute.
     /// This is equivalent to the Python expression 'hasattr(self, attr_name)'.
     #[inline]
-    fn hasattr<N: ?Sized>(&self, attr_name: &N) -> PyResult<'p, bool> where N: ToPyObject<'p> {
+    fn hasattr<N>(&self, attr_name: N) -> PyResult<'p, bool> where N: ToPyObject<'p> {
         attr_name.with_borrowed_ptr(self.python(), |attr_name| unsafe {
             Ok(ffi::PyObject_HasAttr(self.as_ptr(), attr_name) != 0)
         })
@@ -41,7 +39,7 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     /// Retrieves an attribute value.
     /// This is equivalent to the Python expression 'self.attr_name'.
     #[inline]
-    fn getattr<N: ?Sized>(&self, attr_name: &N) -> PyResult<'p, PyObject<'p>> where N: ToPyObject<'p> {
+    fn getattr<N>(&self, attr_name: N) -> PyResult<'p, PyObject<'p>> where N: ToPyObject<'p> {
         let py = self.python();
         attr_name.with_borrowed_ptr(py, |attr_name| unsafe {
             result_from_owned_ptr(py,
@@ -52,7 +50,7 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     /// Sets an attribute value.
     /// This is equivalent to the Python expression 'self.attr_name = value'.
     #[inline]
-    fn setattr<N: ?Sized, V: ?Sized>(&self, attr_name: &N, value: &V) -> PyResult<'p, ()>
+    fn setattr<N, V>(&self, attr_name: N, value: V) -> PyResult<'p, ()>
         where N: ToPyObject<'p>, V: ToPyObject<'p>
     {
         let py = self.python();
@@ -66,7 +64,7 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     /// Deletes an attribute.
     /// This is equivalent to the Python expression 'del self.attr_name'.
     #[inline]
-    fn delattr<N: ?Sized>(&self, attr_name: &N) -> PyResult<'p, ()> where N: ToPyObject<'p> {
+    fn delattr<N>(&self, attr_name: N) -> PyResult<'p, ()> where N: ToPyObject<'p> {
         let py = self.python();
         attr_name.with_borrowed_ptr(py, |attr_name| unsafe {
             error_on_minusone(py,
@@ -74,13 +72,13 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
         })
     }
 
-    /// Compares two python objects.
-    /// This is equivalent to the python expression 'cmp(self, other)'.
-    #[inline]
-    fn compare<O: ?Sized>(&self, other: &O) -> PyResult<'p, Ordering> where O: ToPyObject<'p> {
+    /// Compares two Python objects.
+    /// This is equivalent to the Python expression 'cmp(self, other)'.
+    #[cfg(feature="python27-sys")]
+    fn compare<O>(&self, other: O) -> PyResult<'p, Ordering> where O: ToPyObject<'p> {
         let py = self.python();
         other.with_borrowed_ptr(py, |other| unsafe {
-            let mut result : libc::c_int = std::mem::uninitialized();
+            let mut result : libc::c_int = -1;
             try!(error_on_minusone(py,
                 ffi::PyObject_Cmp(self.as_ptr(), other, &mut result)));
             Ok(if result < 0 {
@@ -94,32 +92,33 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     }
 
     /// Compute the string representation of self.
-    /// This is equivalent to the python expression 'repr(self)'.
+    /// This is equivalent to the Python expression 'repr(self)'.
     #[inline]
     fn repr(&self) -> PyResult<'p, PyObject<'p>> {
         unsafe {
             result_from_owned_ptr(self.python(), ffi::PyObject_Repr(self.as_ptr()))
         }
     }
-    
+
     /// Compute the string representation of self.
-    /// This is equivalent to the python expression 'str(self)'.
+    /// This is equivalent to the Python expression 'str(self)'.
     #[inline]
     fn str(&self) -> PyResult<'p, PyObject<'p>> {
         unsafe {
             result_from_owned_ptr(self.python(), ffi::PyObject_Str(self.as_ptr()))
         }
     }
-    
+
     /// Compute the unicode string representation of self.
-    /// This is equivalent to the python expression 'unistr(self)'.
+    /// This is equivalent to the Python expression 'unistr(self)'.
     #[inline]
+    #[cfg(feature="python27-sys")]
     fn unistr(&self) -> PyResult<'p, PyObject<'p>> {
         unsafe {
             result_from_owned_ptr(self.python(), ffi::PyObject_Unicode(self.as_ptr()))
         }
     }
-    
+
     /// Determines whether this object is callable.
     #[inline]
     fn is_callable(&self) -> bool {
@@ -127,30 +126,30 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
             ffi::PyCallable_Check(self.as_ptr()) != 0
         }
     }
-    
+
     /// Calls the object.
-    /// This is equivalent to the python expression: 'self(*args, **kwargs)'
+    /// This is equivalent to the Python expression: 'self(*args, **kwargs)'
     #[inline]
-    fn call<A: ?Sized>(&self, args: &A, kwargs: Option<&PyDict<'p>>) -> PyResult<'p, PyObject<'p>>
+    fn call<A>(&self, args: A, kwargs: Option<&PyDict<'p>>) -> PyResult<'p, PyObject<'p>>
       where A: ToPyObject<'p, ObjectType=PyTuple<'p>> {
         let py = self.python();
         args.with_borrowed_ptr(py, |args| unsafe {
             result_from_owned_ptr(py, ffi::PyObject_Call(self.as_ptr(), args, kwargs.as_ptr()))
         })
     }
-    
+
     /// Calls a method on the object.
-    /// This is equivalent to the python expression: 'self.name(*args, **kwargs)'
+    /// This is equivalent to the Python expression: 'self.name(*args, **kwargs)'
     #[inline]
-    fn call_method<A: ?Sized>(&self, name: &str, args: &A, kwargs: Option<&PyDict<'p>>) -> PyResult<'p, PyObject<'p>>
+    fn call_method<A>(&self, name: &str, args: A, kwargs: Option<&PyDict<'p>>) -> PyResult<'p, PyObject<'p>>
       where A: ToPyObject<'p, ObjectType=PyTuple<'p>> {
         try!(self.getattr(name)).call(args, kwargs)
     }
-    
+
     /// Retrieves the hash code of the object.
-    /// This is equivalent to the python expression: 'hash(self)'
+    /// This is equivalent to the Python expression: 'hash(self)'
     #[inline]
-    fn hash(&self) -> PyResult<'p, libc::c_long> {
+    fn hash(&self) -> PyResult<'p, ::Py_hash_t> {
         let v = unsafe { ffi::PyObject_Hash(self.as_ptr()) };
         if v == -1 {
             Err(PyErr::fetch(self.python()))
@@ -160,7 +159,7 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     }
     
     /// Returns whether the object is considered to be true.
-    /// This is equivalent to the python expression: 'not not self'
+    /// This is equivalent to the Python expression: 'not not self'
     #[inline]
     fn is_true(&self) -> PyResult<'p, bool> {
         let v = unsafe { ffi::PyObject_IsTrue(self.as_ptr()) };
@@ -172,18 +171,18 @@ pub trait ObjectProtocol<'p> : PythonObject<'p> {
     }
     
     /// Returns the length of the sequence or mapping.
-    /// This is equivalent to the python expression: 'len(self)'
+    /// This is equivalent to the Python expression: 'len(self)'
     #[inline]
-    fn len(&self) -> PyResult<'p, ffi::Py_ssize_t> {
+    fn len(&self) -> PyResult<'p, usize> {
         let v = unsafe { ffi::PyObject_Size(self.as_ptr()) };
         if v == -1 {
             Err(PyErr::fetch(self.python()))
         } else {
-            Ok(v)
+            Ok(v as usize)
         }
     }
     
-    /// This is equivalent to the python expression: 'self[key]'
+    /// This is equivalent to the Python expression: 'self[key]'
     #[inline]
     fn get_item<K>(&self, key: K) -> PyResult<'p, PyObject<'p>> where K: ToPyObject<'p> {
         let py = self.python();
@@ -251,7 +250,7 @@ impl <'p> fmt::Debug for PyObject<'p> {
     fn fmt(&self, f : &mut fmt::Formatter) -> Result<(), fmt::Error> {
         use objectprotocol::ObjectProtocol;
         let repr_obj = try!(self.str().map_err(|_| fmt::Error));
-        let repr = try!(repr_obj.extract::<Cow<str>>().map_err(|_| fmt::Error));
+        let repr = try!(PyString::extract_lossy(&repr_obj).map_err(|_| fmt::Error));
         f.write_str(&*repr)
     }
 }
@@ -260,7 +259,7 @@ impl <'p> fmt::Display for PyObject<'p> {
     fn fmt(&self, f : &mut fmt::Formatter) -> Result<(), fmt::Error> {
         use objectprotocol::ObjectProtocol;
         let repr_obj = try!(self.repr().map_err(|_| fmt::Error));
-        let repr = try!(repr_obj.extract::<Cow<str>>().map_err(|_| fmt::Error));
+        let repr = try!(PyString::extract_lossy(&repr_obj).map_err(|_| fmt::Error));
         f.write_str(&*repr)
     }
 }
