@@ -4,6 +4,9 @@ use err::PyResult;
 use super::PyObject;
 use conversion::{FromPyObject, ToPyObject};
 
+/// Represents a Python `bool`.
+pub struct PyBool<'p>(PyObject<'p>);
+
 pyobject_newtype!(PyBool, PyBool_Check, PyBool_Type);
 
 impl <'p> PyBool<'p> {
@@ -13,12 +16,14 @@ impl <'p> PyBool<'p> {
         if val { py.True() } else { py.False() }
     }
 
+    /// Gets whether this boolean is `true`.
     #[inline]
     pub fn is_true(&self) -> bool {
         self.as_ptr() == unsafe { ::ffi::Py_True() }
     }
 }
 
+/// Converts a rust `bool` to a Python `bool`.
 impl <'p> ToPyObject<'p> for bool {
     type ObjectType = PyBool<'p>;
 
@@ -28,7 +33,7 @@ impl <'p> ToPyObject<'p> for bool {
     }
 
     #[inline]
-    fn with_borrowed_ptr<F, R>(&self, py: Python<'p>, f: F) -> R
+    fn with_borrowed_ptr<F, R>(&self, _py: Python<'p>, f: F) -> R
         where F: FnOnce(*mut ffi::PyObject) -> R
     {
         // Avoid unnecessary Py_INCREF/Py_DECREF pair
@@ -36,8 +41,11 @@ impl <'p> ToPyObject<'p> for bool {
     }
 }
 
-impl <'p, 'a> FromPyObject<'p, 'a> for bool {
-    fn from_py_object(s: &'a PyObject<'p>) -> PyResult<'p, bool> {
+/// Converts a Python `bool` to a rust `bool`.
+///
+/// Fails with `TypeError` if the input is not a Python `bool`.
+impl <'p> FromPyObject<'p> for bool {
+    fn from_py_object(s: &PyObject<'p>) -> PyResult<'p, bool> {
         Ok(try!(s.clone().cast_into::<PyBool>()).is_true())
     }
 }
